﻿using System;
using System.IO;
using System.Diagnostics;
using System.Runtime.InteropServices;
using System.Windows.Forms;

namespace ResChanger
{
    internal class Program
    {
        #region DllImport
        [DllImport("kernel32.dll")]
        static extern bool FreeConsole();
        #endregion

        #region Main
        private static void Main(string[] args)
        {
            FreeConsole();

            if (args.Length != 2)
            {
                ShowMessage("Not enough command line arguments given.");
                return;
            }

            var newResolution = args[0].Split('x');
            int newWidth;
            int newHeight;

            if (newResolution.Length != 2)
            {
                ShowMessage("Invalid resolution format given.");
                return;
            }

            if (!int.TryParse(newResolution[0], out newWidth) || !int.TryParse(newResolution[1], out newHeight))
            {
                ShowMessage("Invalid resolution given, could not parse to integer.");
                return;
            }

            var resolution = new Resolution();
            string supportedModes;

            if (!resolution.IsDisplayModeSupported(newWidth, newHeight, out supportedModes))
            {
                ShowMessage("The mode '" + args[0] + "' is not supported, supported modes are: " + Environment.NewLine + supportedModes +
                            Environment.NewLine);
                return;
            }

            if (!File.Exists(args[1]))
            {
                ShowMessage("The file '" + args[1] + "' does not exists.");
                return;
            }

            if (!resolution.ChangeDisplaySettings(newWidth, newHeight))
                return;

            var runningProgram = Process.Start(args[1]);
            if (runningProgram != null) runningProgram.WaitForExit();

            resolution.RestoreDisplaySettings();
        }
        #endregion

        #region ShowMessage
        private static void ShowMessage(string errorMessage)
        {
            MessageBox.Show("Invalid command line arguments given:" + Environment.NewLine + Environment.NewLine +
                "Error: " + errorMessage + Environment.NewLine + Environment.NewLine +
                "Expected \"<resolution>\" \"<program to start, with full path\">" + Environment.NewLine +
                Environment.NewLine +
                "For example: " + Path.GetFileName(Environment.GetCommandLineArgs()[0]) +
                " \"1920x1080\"  \"C:\\Program Files (x86)\\SQUARE ENIX\\Tombraider\\TombRaider.exe\"" +
                Environment.NewLine + Environment.NewLine +
                "This will first switch the resolution to 1920x1080 and then start Tombraider." +
                Environment.NewLine +
                "After you exit the game the resolution will be switched back to the old configuration.", "Reschanger v1.1");
        }
        #endregion
    }
}